@@ -55,13 +55,13 @@ static void rpmTSIFree(tsortInfo tsi)
     while (tsi->tsi_relations != NULL) {
 	rel = tsi->tsi_relations;
 	tsi->tsi_relations = tsi->tsi_relations->rel_next;
-	rel = _free(rel);
+	free(rel);
     }
     while (tsi->tsi_forward_relations != NULL) {
 	rel = tsi->tsi_forward_relations;
 	tsi->tsi_forward_relations = \
 	    tsi->tsi_forward_relations->rel_next;
-	rel = _free(rel);
+	free(rel);
     }
 }
 
@@ -208,6 +208,8 @@ static void addQ(tsortInfo p, tsortInfo * qp, tsortInfo * rp,
 		rpm_color_t prefcolor)
 {
     tsortInfo q, qprev;
+    rpm_color_t pcolor = rpmteColor(p->te);
+    int tailcond;
 
     /* Mark the package as queued. */
     p->tsi_reqx = 1;
@@ -218,13 +220,18 @@ static void addQ(tsortInfo p, tsortInfo * qp, tsortInfo * rp,
 	return;
     }
 
-    /* Find location in queue using metric tsi_qcnt. */
+    if (rpmteType(p->te) == TR_ADDED)
+	tailcond = (pcolor && pcolor != prefcolor);
+    else
+	tailcond = (pcolor && pcolor == prefcolor);
+
+    /* Find location in queue using metric tsi_qcnt and color. */
     for (qprev = NULL, q = (*qp);
 	 q != NULL;
 	 qprev = q, q = q->tsi_suc)
     {
-	/* XXX Insure preferred color first. */
-	if (rpmteColor(p->te) != prefcolor && rpmteColor(p->te) != rpmteColor(q->te))
+	/* Place preferred color towards queue head on install, tail on erase */
+	if (tailcond && (pcolor != rpmteColor(q->te)))
 	    continue;
 
 	if (q->tsi_qcnt <= p->tsi_qcnt)
@@ -251,7 +258,8 @@ typedef struct sccData_s {
     int sccCnt;			/* Number of SCC's found */
 } * sccData;
 
-static void tarjan(sccData sd, tsortInfo tsi) {
+static void tarjan(sccData sd, tsortInfo tsi)
+{
     tsortInfo tsi_q;
     relation rel;
 
@@ -344,8 +352,9 @@ static scc detectSCCs(tsortInfo orderInfo, int nelem, int debugloops)
 	int msglvl = debugloops ?  RPMLOG_WARNING : RPMLOG_DEBUG;
 	rpmlog(msglvl, "%i Strongly Connected Components\n", sd.sccCnt-2);
 	for (int i = 2; i < sd.sccCnt; i++) {
-	    rpmlog(msglvl, "SCC #%i: requires %i packages\n",
-		   i, SCCs[i].count);
+	    rpmlog(msglvl, "SCC #%i: %i members (%i external dependencies)\n",
+			   i-1, SCCs[i].size, SCCs[i].count);
+
 	    /* loop over members */
 	    for (int j = 0; j < SCCs[i].size; j++) {
 		tsortInfo member = SCCs[i].members[j];
@@ -421,38 +430,21 @@ static void collectTE(rpm_color_t prefcolor, tsortInfo q,
     q->tsi_SccIdx = 0;
 }
 
-static void collectSCC(rpm_color_t prefcolor, tsortInfo p_tsi,
-		       rpmte * newOrder, int * newOrderCount,
-		       scc SCCs, tsortInfo * queue_end)
+static void dijkstra(const struct scc_s *SCC, int sccNr)
 {
-    int sccNr = p_tsi->tsi_SccIdx;
-    struct scc_s SCC = SCCs[sccNr];
-    int i;
     int start, end;
     relation rel;
 
-    /* remove p from the outer queue */
-    tsortInfo outer_queue_start = p_tsi->tsi_suc;
-    p_tsi->tsi_suc = NULL;
-
-    /*
-     * Run a multi source Dijkstra's algorithm to find relations
-     * that can be zapped with least danger to pre reqs.
-     * As weight of the edges is always 1 it is not necessary to
-     * sort the vertices by distance as the queue gets them
-     * already in order
-    */
-
     /* can use a simple queue as edge weights are always 1 */
-    tsortInfo * queue = xmalloc((SCC.size+1) * sizeof(*queue));
+    tsortInfo * queue = xmalloc((SCC->size+1) * sizeof(*queue));
 
     /*
      * Find packages that are prerequired and use them as
      * starting points for the Dijkstra algorithm
      */
     start = end = 0;
-    for (i = 0; i < SCC.size; i++) {
-	tsortInfo tsi = SCC.members[i];
+    for (int i = 0; i < SCC->size; i++) {
+	tsortInfo tsi = SCC->members[i];
 	tsi->tsi_SccLowlink = INT_MAX;
 	for (rel=tsi->tsi_forward_relations; rel != NULL; rel=rel->rel_next) {
 	    if (rel->rel_flags && rel->rel_suc->tsi_SccIdx == sccNr) {
@@ -468,8 +460,8 @@ static void collectSCC(rpm_color_t prefcolor, tsortInfo p_tsi,
     }
 
     if (start == end) { /* no regular prereqs; add self prereqs to queue */
-	for (i = 0; i < SCC.size; i++) {
-	    tsortInfo tsi = SCC.members[i];
+	for (int i = 0; i < SCC->size; i++) {
+	    tsortInfo tsi = SCC->members[i];
 	    if (tsi->tsi_SccLowlink != INT_MAX) {
 		queue[end++] = tsi;
 	    }
@@ -488,8 +480,28 @@ static void collectSCC(rpm_color_t prefcolor, tsortInfo p_tsi,
 	    }
 	}
     }
-    queue = _free(queue);
+    free(queue);
+}
 
+static void collectSCC(rpm_color_t prefcolor, tsortInfo p_tsi,
+		       rpmte * newOrder, int * newOrderCount,
+		       scc SCCs, tsortInfo * queue_end)
+{
+    int sccNr = p_tsi->tsi_SccIdx;
+    const struct scc_s * SCC = SCCs+sccNr;
+
+    /* remove p from the outer queue */
+    tsortInfo outer_queue_start = p_tsi->tsi_suc;
+    p_tsi->tsi_suc = NULL;
+
+    /*
+     * Run a multi source Dijkstra's algorithm to find relations
+     * that can be zapped with least danger to pre reqs.
+     * As weight of the edges is always 1 it is not necessary to
+     * sort the vertices by distance as the queue gets them
+     * already in order
+    */
+    dijkstra(SCC, sccNr);
 
     while (1) {
 	tsortInfo best = NULL;
@@ -497,8 +509,8 @@ static void collectSCC(rpm_color_t prefcolor, tsortInfo p_tsi,
 	int best_score = 0;
 
 	/* select best candidate to start with */
-	for (int i = 0; i < SCC.size; i++) {
-	    tsortInfo tsi = SCC.members[i];
+	for (int i = 0; i < SCC->size; i++) {
+	    tsortInfo tsi = SCC->members[i];
 	    if (tsi->tsi_SccIdx == 0) /* package already collected */
 		continue;
 	    if (tsi->tsi_SccLowlink >= best_score) {
@@ -536,7 +548,8 @@ int rpmtsOrder(rpmts ts)
     rpmte * newOrder;
     int newOrderCount = 0;
     int rc;
-    rpmal erasedPackages = rpmalCreate(5, rpmtsColor(ts), prefcolor);
+    rpmal erasedPackages = rpmalCreate(5, rpmtsFlags(ts),
+					rpmtsColor(ts), prefcolor);
     scc SCCs;
     int nelem = rpmtsNElements(ts);
     tsortInfo sortInfo = xcalloc(nelem, sizeof(struct tsortInfo_s));
@@ -549,7 +562,7 @@ int rpmtsOrder(rpmts ts)
     while ((p = rpmtsiNext(pi, TR_REMOVED)) != NULL) {
         rpmalAdd(erasedPackages, p);
     }
-    pi = rpmtsiFree(pi);
+    rpmtsiFree(pi);
 
     for (int i = 0; i < nelem; i++) {
 	sortInfo[i].te = tsmem->order[i];
@@ -579,7 +592,7 @@ int rpmtsOrder(rpmts ts)
     }
 
     seenColls = argvFree(seenColls);
-    pi = rpmtsiFree(pi);
+    rpmtsiFree(pi);
 
     newOrder = xcalloc(tsmem->orderCount, sizeof(*newOrder));
     SCCs = detectSCCs(sortInfo, nelem, (rpmtsFlags(ts) & RPMTRANS_FLAG_DEPLOOPS));
