@@ -6,10 +6,6 @@
 #include "system.h"
 const char *__progname;
 
-#if HAVE_MCHECK_H
-#include <mcheck.h>
-#endif
-
 #include <rpm/rpmcli.h>
 #include <rpm/rpmlib.h>		/* rpmEVR, rpmReadConfigFiles etc */
 #include <rpm/rpmlog.h>
@@ -108,14 +104,14 @@ static void rpmcliAllArgCallback( poptContext con,
 	rpmcliConfigured();
 	(void) rpmDefineMacro(NULL, t, RMIL_CMDLINE);
 	(void) rpmDefineMacro(rpmCLIMacroContext, t, RMIL_CMDLINE);
-	s = _free(s);
+	free(s);
 	break;
     }
     case 'E':
 	rpmcliConfigured();
 	{   char *val = rpmExpand(arg, NULL);
 	    fprintf(stdout, "%s\n", val);
-	    val = _free(val);
+	    free(val);
 	}
 	break;
     case POPT_DBPATH:
@@ -157,7 +153,7 @@ struct poptOption rpmcliAllPoptTable[] = {
  { "debug", 'd', POPT_ARG_VAL|POPT_ARGFLAG_DOC_HIDDEN, &_debug, -1,
         NULL, NULL },
 
- { "predefine", 'D', POPT_ARG_STRING|POPT_ARGFLAG_DOC_HIDDEN, 0, POPT_PREDEFINE,
+ { "predefine", '\0', POPT_ARG_STRING|POPT_ARGFLAG_DOC_HIDDEN, 0, POPT_PREDEFINE,
 	N_("predefine MACRO with value EXPR"),
 	N_("'MACRO EXPR'") },
  { "define", 'D', POPT_ARG_STRING, 0, 'D',
@@ -226,10 +222,7 @@ rpmcliFini(poptContext optCon)
     rpmFreeMacros(rpmCLIMacroContext);
     rpmFreeRpmrc();
     rpmlogClose();
-
-#if HAVE_MCHECK_H && HAVE_MTRACE
-    muntrace();   /* Trace malloc only if MALLOC_TRACE=mtrace-output-file. */
-#endif
+    rpmcliInitialized = -1;
 
     return NULL;
 }
@@ -237,14 +230,10 @@ rpmcliFini(poptContext optCon)
 poptContext
 rpmcliInit(int argc, char *const argv[], struct poptOption * optionsTable)
 {
-    const char * optArg;
     poptContext optCon;
     int rc;
     const char *ctx, *execPath;
 
-#if HAVE_MCHECK_H && HAVE_MTRACE
-    mtrace();   /* Trace malloc only if MALLOC_TRACE=mtrace-output-file. */
-#endif
     setprogname(argv[0]);       /* Retrofit glibc __progname */
 
     /* XXX glibc churn sanity */
@@ -285,15 +274,9 @@ rpmcliInit(int argc, char *const argv[], struct poptOption * optionsTable)
 
     /* Process all options, whine if unknown. */
     while ((rc = poptGetNextOpt(optCon)) > 0) {
-	optArg = poptGetOptArg(optCon);
-	switch (rc) {
-	default:
-	    fprintf(stderr, _("%s: option table misconfigured (%d)\n"),
+	fprintf(stderr, _("%s: option table misconfigured (%d)\n"),
 		__progname, rc);
-	    exit(EXIT_FAILURE);
-
-	    break;
-        }
+	exit(EXIT_FAILURE);
     }
 
     if (rc < -1) {
