@@ -5,6 +5,8 @@
 
 #include "system.h"
 
+#include <ctype.h>
+
 #include <rpm/rpmlib.h>			/* RPMSIGTAG & related */
 #include <rpm/rpmpgp.h>
 #include <rpm/rpmcli.h>
@@ -15,7 +17,6 @@
 #include <rpm/rpmstring.h>
 #include <rpm/rpmkeyring.h>
 
-#include "rpmio/digest.h"
 #include "rpmio/rpmio_internal.h" 	/* fdSetBundle() */
 #include "lib/rpmlead.h"
 #include "lib/signature.h"
@@ -103,17 +104,18 @@ int rpmcliImportPubkeys(rpmts ts, ARGV_const_t argv)
 /**
  * @todo If the GPG key was known available, the md5 digest could be skipped.
  */
-static int readFile(FD_t fd, const char * fn, pgpDig dig,
+static int readFile(FD_t fd, const char * fn,
 		    rpmDigestBundle plbundle, rpmDigestBundle hdrbundle)
 {
     unsigned char buf[4*BUFSIZ];
     ssize_t count;
     int rc = 1;
     Header h = NULL;
+    char *msg = NULL;
 
     /* Read the header from the package. */
-    if ((h = headerRead(fd, HEADER_MAGIC_YES)) == NULL) {
-	rpmlog(RPMLOG_ERR, _("%s: headerRead failed\n"), fn);
+    if (rpmReadHeader(NULL, fd, &h, &msg) != RPMRC_OK) {
+	rpmlog(RPMLOG_ERR, _("%s: headerRead failed: %s\n"), fn, msg);
 	goto exit;
     }
 
@@ -141,27 +143,11 @@ static int readFile(FD_t fd, const char * fn, pgpDig dig,
     rc = 0;
 
 exit:
+    free(msg);
     headerFree(h);
     return rc;
 }
 
-/* Parse the parameters from the OpenPGP packets that will be needed. */
-/* XXX TODO: unify with similar parsePGP() in package.c */
-static rpmRC parsePGP(rpmtd sigtd, const char *fn, pgpDig dig)
-{
-    rpmRC rc = RPMRC_FAIL;
-    int debug = (_print_pkts & rpmIsDebug());
-    if ((pgpPrtPkts(sigtd->data, sigtd->count, dig, debug) == 0) &&
-	 (dig->signature.version == 3 || dig->signature.version == 4)) {
-	rc = RPMRC_OK;
-    } else {
-	rpmlog(RPMLOG_ERR,
-	    _("skipping package %s with unverifiable V%u signature\n"), fn,
-	    dig->signature.version);
-    }
-    return rc;
-}
-
 /* 
  * Figure best available signature. 
  * XXX TODO: Similar detection in rpmReadPackageFile(), unify these.
@@ -264,8 +250,7 @@ static int rpmpkgVerifySigs(rpmKeyring keyring, rpmQueryFlags flags,
     char *untrustedKeys = NULL;
     struct rpmtd_s sigtd;
     rpmTagVal sigtag;
-    pgpDig dig = NULL;
-    pgpDigParams sigp;
+    pgpDigParams sig = NULL;
     Header sigh = NULL;
     HeaderIterator hi = NULL;
     char * msg = NULL;
@@ -277,16 +262,9 @@ static int rpmpkgVerifySigs(rpmKeyring keyring, rpmQueryFlags flags,
     rpmDigestBundle plbundle = rpmDigestBundleNew();
     rpmDigestBundle hdrbundle = rpmDigestBundleNew();
 
-    rpmlead lead = rpmLeadNew();
-    if ((rc = rpmLeadRead(fd, lead)) == RPMRC_OK) {
-	const char *lmsg = NULL;
-	rc = rpmLeadCheck(lead, &lmsg);
-	if (rc != RPMRC_OK) 
-	    rpmlog(RPMLOG_ERR, "%s: %s\n", fn, lmsg);
-    }
-    lead = rpmLeadFree(lead);
-
-    if (rc != RPMRC_OK) {
+    if ((rc = rpmLeadRead(fd, NULL, NULL, &msg)) != RPMRC_OK) {
+	rpmlog(RPMLOG_ERR, "%s: %s\n", fn, msg);
+	free(msg);
 	goto exit;
     }
 
@@ -310,22 +288,20 @@ static int rpmpkgVerifySigs(rpmKeyring keyring, rpmQueryFlags flags,
     /* Grab a hint of what needs doing to avoid duplication. */
     sigtag = bestSig(sigh, nosignatures, nodigests);
 
-    dig = pgpNewDig();
-    sigp = &dig->signature;
-
     /* XXX RSA needs the hash_algo, so decode early. */
     if (sigtag == RPMSIGTAG_RSA || sigtag == RPMSIGTAG_PGP ||
 		sigtag == RPMSIGTAG_DSA || sigtag == RPMSIGTAG_GPG) {
-	int xx = -1;
+	unsigned int hashalgo;
 	if (headerGet(sigh, sigtag, &sigtd, HEADERGET_DEFAULT)) {
-	    xx = pgpPrtPkts(sigtd.data, sigtd.count, dig, 0);
+	    parsePGPSig(&sigtd, "package", fn, &sig);
 	    rpmtdFreeData(&sigtd);
 	}
-	if (xx) goto exit;
+	if (sig == NULL) goto exit;
 	    
 	/* XXX assume same hash_algo in header-only and header+payload */
-	rpmDigestBundleAdd(plbundle, sigp->hash_algo, RPMDIGEST_NONE);
-	rpmDigestBundleAdd(hdrbundle, sigp->hash_algo, RPMDIGEST_NONE);
+	hashalgo = pgpDigParamsAlgo(sig, PGPVAL_HASHALGO);
+	rpmDigestBundleAdd(plbundle, hashalgo, RPMDIGEST_NONE);
+	rpmDigestBundleAdd(hdrbundle, hashalgo, RPMDIGEST_NONE);
     }
 
     if (headerIsEntry(sigh, RPMSIGTAG_PGP) ||
@@ -342,7 +318,7 @@ static int rpmpkgVerifySigs(rpmKeyring keyring, rpmQueryFlags flags,
 
     /* Read the file, generating digest(s) on the fly. */
     fdSetBundle(fd, plbundle);
-    if (readFile(fd, fn, dig, plbundle, hdrbundle)) {
+    if (readFile(fd, fn, plbundle, hdrbundle)) {
 	goto exit;
     }
 
@@ -357,7 +333,7 @@ static int rpmpkgVerifySigs(rpmKeyring keyring, rpmQueryFlags flags,
 	    continue;
 
 	/* Clean up parameters from previous sigtag. */
-	pgpCleanDig(dig);
+	sig = pgpDigParamsFree(sig);
 
 	switch (sigtd.tag) {
 	case RPMSIGTAG_GPG:
@@ -368,11 +344,10 @@ static int rpmpkgVerifySigs(rpmKeyring keyring, rpmQueryFlags flags,
 	case RPMSIGTAG_DSA:
 	    if (nosignatures)
 		 continue;
-	    if (parsePGP(&sigtd, fn, dig) != RPMRC_OK) {
+	    if (parsePGPSig(&sigtd, "package", fn, &sig))
 		goto exit;
-	    }
 	    ctx = rpmDigestBundleDupCtx(havekey ? plbundle : hdrbundle,
-					dig->signature.hash_algo);
+					pgpDigParamsAlgo(sig, PGPVAL_HASHALGO));
 	    break;
 	case RPMSIGTAG_SHA1:
 	    if (nodigests)
@@ -389,7 +364,7 @@ static int rpmpkgVerifySigs(rpmKeyring keyring, rpmQueryFlags flags,
 	    break;
 	}
 
-	rc = rpmVerifySignature(keyring, &sigtd, dig, ctx, &result);
+	rc = rpmVerifySignature(keyring, &sigtd, sig, ctx, &result);
 	rpmDigestFinal(ctx, NULL, NULL, 0);
 
 	formatResult(sigtd.tag, rc, result, havekey, 
@@ -426,7 +401,7 @@ exit:
     fdSetBundle(fd, NULL); /* XXX avoid double-free from fd close */
     sigh = rpmFreeSignature(sigh);
     hi = headerFreeIterator(hi);
-    pgpFreeDig(dig);
+    pgpDigParamsFree(sig);
     return res;
 }
 
