@@ -199,19 +199,28 @@ static void buildRelocs(rpmte p, Header h, rpmRelocation *relocs)
  * @param key		(TR_ADDED) package retrieval key (e.g. file name)
  * @param relocs	(TR_ADDED) package file relocations
  */
-static void addTE(rpmte p, Header h, fnpyKey key, rpmRelocation * relocs)
+static int addTE(rpmte p, Header h, fnpyKey key, rpmRelocation * relocs)
 {
     struct rpmtd_s colls;
+    int rc = 1; /* assume failure */
 
     p->name = headerGetAsString(h, RPMTAG_NAME);
     p->version = headerGetAsString(h, RPMTAG_VERSION);
     p->release = headerGetAsString(h, RPMTAG_RELEASE);
 
+    /* name, version and release are required in all packages */
+    if (p->name == NULL || p->version == NULL || p->release == NULL)
+	goto exit;
+
     p->epoch = headerGetAsString(h, RPMTAG_EPOCH);
 
     p->arch = headerGetAsString(h, RPMTAG_ARCH);
     p->os = headerGetAsString(h, RPMTAG_OS);
 
+    /* gpg-pubkey's dont have os or arch (sigh), for others they are required */
+    if (!rstreq(p->name, "gpg-pubkey") && (p->arch == NULL || p->os == NULL))
+	goto exit;
+
     p->isSource = headerIsSource(h);
     
     p->NEVR = headerGetAsString(h, RPMTAG_NEVR);
@@ -239,11 +248,15 @@ static void addTE(rpmte p, Header h, fnpyKey key, rpmRelocation * relocs)
     p->fs = rpmfsNew(h, p->type);
     p->fi = getFI(p, h);
 
+    /* Packages with no files return an empty file info set, NULL is an error */
+    if (p->fi == NULL)
+	goto exit;
+
     /* See if we have pre/posttrans scripts. */
-    p->transscripts |= (headerIsEntry(h, RPMTAG_PRETRANS) &&
+    p->transscripts |= (headerIsEntry(h, RPMTAG_PRETRANS) ||
 			 headerIsEntry(h, RPMTAG_PRETRANSPROG)) ?
 			RPMTE_HAVE_PRETRANS : 0;
-    p->transscripts |= (headerIsEntry(h, RPMTAG_POSTTRANS) &&
+    p->transscripts |= (headerIsEntry(h, RPMTAG_POSTTRANS) ||
 			 headerIsEntry(h, RPMTAG_POSTTRANSPROG)) ?
 			RPMTE_HAVE_POSTTRANS : 0;
 
@@ -262,7 +275,14 @@ static void addTE(rpmte p, Header h, fnpyKey key, rpmRelocation * relocs)
 
     rpmteColorDS(p, RPMTAG_PROVIDENAME);
     rpmteColorDS(p, RPMTAG_REQUIRENAME);
-    return;
+
+    if (p->type == TR_ADDED)
+	p->pkgFileSize = headerGetNumber(h, RPMTAG_LONGSIGSIZE) + 96 + 256;
+
+    rc = 0;
+
+exit:
+    return rc;
 }
 
 rpmte rpmteFree(rpmte te)
@@ -309,14 +329,10 @@ rpmte rpmteNew(rpmts ts, Header h, rpmElementType type, fnpyKey key,
     rpmte p = xcalloc(1, sizeof(*p));
     p->ts = ts;
     p->type = type;
-    addTE(p, h, key, relocs);
-    switch (type) {
-    case TR_ADDED:
-	p->pkgFileSize = headerGetNumber(h, RPMTAG_LONGSIGSIZE) + 96 + 256;
-	break;
-    case TR_REMOVED:
-	/* nothing to do */
-	break;
+
+    if (addTE(p, h, key, relocs)) {
+	rpmteFree(p);
+	return NULL;
     }
 
     return p;
@@ -449,7 +465,8 @@ rpm_loff_t rpmtePkgFileSize(rpmte te)
     return (te != NULL ? te->pkgFileSize : 0);
 }
 
-unsigned int rpmteHeaderSize(rpmte te) {
+unsigned int rpmteHeaderSize(rpmte te)
+{
     return (te != NULL ? te->headerSize : 0);
 }
 
@@ -478,7 +495,8 @@ void rpmteSetTSI(rpmte te, tsortInfo tsi)
     te->tsi = tsi;
 }
 
-void rpmteSetDependsOn(rpmte te, rpmte depends) {
+void rpmteSetDependsOn(rpmte te, rpmte depends)
+{
     te->depends = depends;
 }
 
@@ -627,7 +645,7 @@ static Header rpmteDBHeader(rpmte te)
     /* iterator returns weak refs, grab hold of header */
     if ((h = rpmdbNextIterator(mi)))
 	h = headerLink(h);
-    mi = rpmdbFreeIterator(mi);
+    rpmdbFreeIterator(mi);
     return h;
 }
 
@@ -658,6 +676,7 @@ static Header rpmteFDHeader(rpmte te)
 
 static int rpmteOpen(rpmte te, int reload_fi)
 {
+    int rc = 0; /* assume failure */
     Header h = NULL;
     if (te == NULL || te->ts == NULL || rpmteFailed(te))
 	goto exit;
@@ -674,7 +693,11 @@ static int rpmteOpen(rpmte te, int reload_fi)
     }
     if (h != NULL) {
 	if (reload_fi) {
+	    /* This can fail if we get a different, bad header from callback */
 	    te->fi = getFI(te, h);
+	    rc = (te->fi != NULL);
+	} else {
+	    rc = 1;
 	}
 	
 	rpmteSetHeader(te, h);
@@ -682,7 +705,7 @@ static int rpmteOpen(rpmte te, int reload_fi)
     }
 
 exit:
-    return (h != NULL);
+    return rc;
 }
 
 static int rpmteClose(rpmte te, int reset_fi)
@@ -823,7 +846,8 @@ const char * rpmteTypeString(rpmte te)
     }
 }
 
-rpmfs rpmteGetFileStates(rpmte te) {
+rpmfs rpmteGetFileStates(rpmte te)
+{
     return te->fs;
 }
 
@@ -894,7 +918,8 @@ int rpmteProcess(rpmte te, pkgGoal goal)
 {
     /* Only install/erase resets pkg file info */
     int scriptstage = (goal != PKG_INSTALL && goal != PKG_ERASE);
-    int reset_fi = (scriptstage == 0);
+    int test = (rpmtsFlags(te->ts) & RPMTRANS_FLAG_TEST);
+    int reset_fi = (scriptstage == 0 && test == 0);
     int failed = 1;
 
     /* Dont bother opening for elements without pre/posttrans scripts */
@@ -918,8 +943,7 @@ int rpmteProcess(rpmte te, pkgGoal goal)
 	rpmteRunAllCollections(te, PLUGINHOOK_COLL_POST_ANY);
     }
 
-    /* XXX should %pretrans failure fail the package install? */
-    if (failed && !scriptstage) {
+    if (failed) {
 	failed = rpmteMarkFailed(te);
     }
 
