@@ -13,6 +13,7 @@
 #include <rpm/rpmfileutil.h>		/* rpmtsOpenDB() needs rpmGetPath */
 #include <rpm/rpmstring.h>
 #include <rpm/rpmkeyring.h>
+#include <rpm/rpmbase64.h>
 
 #include <rpm/rpmdb.h>
 #include <rpm/rpmds.h>
@@ -88,9 +89,8 @@ int rpmtsOpenDB(rpmts ts, int dbmode)
     rc = rpmdbOpen(ts->rootDir, &ts->rdb, ts->dbmode, 0644);
     if (rc) {
 	char * dn = rpmGetPath(ts->rootDir, "%{_dbpath}", NULL);
-	rpmlog(RPMLOG_ERR,
-			_("cannot open Packages database in %s\n"), dn);
-	dn = _free(dn);
+	rpmlog(RPMLOG_ERR, _("cannot open Packages database in %s\n"), dn);
+	free(dn);
     }
     return rc;
 }
@@ -161,7 +161,6 @@ rpmdbMatchIterator rpmtsInitIterator(const rpmts ts, rpmDbiTagVal rpmtag,
     rpmdbMatchIterator mi = NULL;
     const char * arch = NULL;
     char *tmp = NULL;
-    int xx;
 
     if (ts == NULL)
 	return NULL;
@@ -237,7 +236,7 @@ rpmdbMatchIterator rpmtsInitIterator(const rpmts ts, rpmDbiTagVal rpmtag,
 
     /* Select specified arch only. */
     if (arch != NULL)
-	xx = rpmdbSetIteratorRE(mi, RPMTAG_ARCH, RPMMIRE_DEFAULT, arch);
+	rpmdbSetIteratorRE(mi, RPMTAG_ARCH, RPMMIRE_DEFAULT, arch);
 
 exit:
     free(tmp);
@@ -259,11 +258,7 @@ rpmKeyring rpmtsGetKeyring(rpmts ts, int autoload)
 
 int rpmtsSetKeyring(rpmts ts, rpmKeyring keyring)
 {
-    /*
-     * Should we permit switching keyring on the fly? For now, require
-     * rpmdb isn't open yet (fairly arbitrary limitation)...
-     */
-    if (ts == NULL || rpmtsGetRdb(ts) != NULL)
+    if (ts == NULL)
 	return -1;
 
     rpmKeyringFree(ts->keyring);
@@ -321,7 +316,7 @@ static int loadKeyringFromDB(rpmts ts)
 	    uint8_t *pkt;
 	    size_t pktlen;
 
-	    if (b64decode(key, (void **) &pkt, &pktlen) == 0) {
+	    if (rpmBase64Decode(key, (void **) &pkt, &pktlen) == 0) {
 		rpmPubkey key = rpmPubkeyNew(pkt, pktlen);
 		if (rpmKeyringAddKey(ts->keyring, key) == 0) {
 		    char *nvr = headerGetAsString(h, RPMTAG_NVR);
@@ -342,24 +337,30 @@ static int loadKeyringFromDB(rpmts ts)
 
 static void loadKeyring(rpmts ts)
 {
-    ts->keyring = rpmKeyringNew();
-    if (loadKeyringFromFiles(ts) == 0) {
-	if (loadKeyringFromDB(ts) > 0) {
-	    /* XXX make this a warning someday... */
-	    rpmlog(RPMLOG_DEBUG, "Using legacy gpg-pubkey(s) from rpmdb\n");
+    /* Never load the keyring if signature checking is disabled */
+    if ((rpmtsVSFlags(ts) & _RPMVSF_NOSIGNATURES) != _RPMVSF_NOSIGNATURES) {
+	ts->keyring = rpmKeyringNew();
+	if (loadKeyringFromFiles(ts) == 0) {
+	    if (loadKeyringFromDB(ts) > 0) {
+		/* XXX make this a warning someday... */
+		rpmlog(RPMLOG_DEBUG, "Using legacy gpg-pubkey(s) from rpmdb\n");
+	    }
 	}
     }
 }
 
 /* Build pubkey header. */
-static int makePubkeyHeader(rpmts ts, rpmPubkey key, Header h)
+static int makePubkeyHeader(rpmts ts, rpmPubkey key, Header * hdrp)
 {
+    Header h = headerNew();
     const char * afmt = "%{pubkeys:armor}";
     const char * group = "Public Keys";
     const char * license = "pubkey";
     const char * buildhost = "localhost";
+    const char * userid;
     rpmsenseFlags pflags = (RPMSENSE_KEYRING|RPMSENSE_EQUAL);
     uint32_t zero = 0;
+    uint32_t keytime = 0;
     pgpDig dig = NULL;
     pgpDigParams pubp = NULL;
     char * d = NULL;
@@ -375,14 +376,17 @@ static int makePubkeyHeader(rpmts ts, rpmPubkey key, Header h)
 	goto exit;
     if ((dig = rpmPubkeyDig(key)) == NULL)
 	goto exit;
+    if ((pubp = pgpDigGetParams(dig, PGPTAG_PUBLIC_KEY)) == NULL)
+	goto exit;
 
     /* Build header elements. */
-    pubp = &dig->pubkey;
     v = pgpHexStr(pubp->signid, sizeof(pubp->signid)); 
     r = pgpHexStr(pubp->time, sizeof(pubp->time));
+    userid = pubp->userid ? pubp->userid : "none";
+    keytime = pgpGrab(pubp->time, sizeof(pubp->time));
 
     rasprintf(&n, "gpg(%s)", v+8);
-    rasprintf(&u, "gpg(%s)", pubp->userid ? pubp->userid : "none");
+    rasprintf(&u, "gpg(%s)", userid);
     rasprintf(&evr, "%d:%s-%s", pubp->version, v, r);
 
     headerPutString(h, RPMTAG_PUBKEYS, enc);
@@ -397,6 +401,7 @@ static int makePubkeyHeader(rpmts ts, rpmPubkey key, Header h)
     headerPutString(h, RPMTAG_GROUP, group);
     headerPutString(h, RPMTAG_LICENSE, license);
     headerPutString(h, RPMTAG_SUMMARY, u);
+    headerPutString(h, RPMTAG_PACKAGER, userid);
 
     headerPutUint32(h, RPMTAG_SIZE, &zero, 1);
 
@@ -410,16 +415,32 @@ static int makePubkeyHeader(rpmts ts, rpmPubkey key, Header h)
 
     headerPutString(h, RPMTAG_RPMVERSION, RPMVERSION);
     headerPutString(h, RPMTAG_BUILDHOST, buildhost);
+    headerPutUint32(h, RPMTAG_BUILDTIME, &keytime, 1);
     headerPutString(h, RPMTAG_SOURCERPM, "(none)");
 
-    {   rpm_tid_t tid = rpmtsGetTid(ts);
-	headerPutUint32(h, RPMTAG_INSTALLTIME, &tid, 1);
-	headerPutUint32(h, RPMTAG_INSTALLTID, &tid, 1);
-	headerPutUint32(h, RPMTAG_BUILDTIME, &tid, 1);
+    /* Reload the lot to immutable region and stomp sha1 digest on it */
+    h = headerReload(h, RPMTAG_HEADERIMMUTABLE);
+    if (h != NULL) {
+	char *sha1 = NULL;
+	unsigned int blen = 0;
+	const void *blob = headerExport(h, &blen);
+
+	/* XXX FIXME: bah, this code is repeated in way too many places */
+	DIGEST_CTX ctx = rpmDigestInit(PGPHASHALGO_SHA1, RPMDIGEST_NONE);
+	rpmDigestUpdate(ctx, rpm_header_magic, sizeof(rpm_header_magic));
+	rpmDigestUpdate(ctx, blob, blen);
+	rpmDigestFinal(ctx, (void **)&sha1, NULL, 1);
+
+	if (sha1) {
+	    headerPutString(h, RPMTAG_SHA1HEADER, sha1);
+	    *hdrp = headerLink(h);
+	    rc = 0;
+	}
+	free(sha1);
     }
-    rc = 0;
 
 exit:
+    headerFree(h);
     pgpFreeDig(dig);
     free(n);
     free(u);
@@ -434,7 +455,7 @@ exit:
 
 rpmRC rpmtsImportPubkey(const rpmts ts, const unsigned char * pkt, size_t pktlen)
 {
-    Header h = headerNew();
+    Header h = NULL;
     rpmRC rc = RPMRC_FAIL;		/* assume failure */
     rpmPubkey pubkey = NULL;
     rpmKeyring keyring = rpmtsGetKeyring(ts, 1);
@@ -448,14 +469,21 @@ rpmRC rpmtsImportPubkey(const rpmts ts, const unsigned char * pkt, size_t pktlen
 
     /* If we dont already have the key, make a persistent record of it */
     if (krc == 0) {
-	if (makePubkeyHeader(ts, pubkey, h) != 0) 
+	rpm_tid_t tid = rpmtsGetTid(ts);
+
+	if (makePubkeyHeader(ts, pubkey, &h) != 0) 
 	    goto exit;
 
+	headerPutUint32(h, RPMTAG_INSTALLTIME, &tid, 1);
+	headerPutUint32(h, RPMTAG_INSTALLTID, &tid, 1);
+
 	/* Add header to database. */
-	if (rpmtsOpenDB(ts, (O_RDWR|O_CREAT)))
-	    goto exit;
-	if (rpmdbAdd(rpmtsGetRdb(ts), h) != 0)
-	    goto exit;
+	if (!(rpmtsFlags(ts) & RPMTRANS_FLAG_TEST)) {
+	    if (rpmtsOpenDB(ts, (O_RDWR|O_CREAT)))
+		goto exit;
+	    if (rpmdbAdd(rpmtsGetRdb(ts), h) != 0)
+		goto exit;
+	}
     }
     rc = RPMRC_OK;
 
@@ -500,7 +528,7 @@ rpmps rpmtsProblems(rpmts ts)
 	rpmpsMerge(ps, teprobs);
 	rpmpsFree(teprobs);
     }
-    pi = rpmtsiFree(pi);
+    rpmtsiFree(pi);
 
     /* Return NULL on no problems instead of an empty set */
     if (rpmpsNumProblems(ps) == 0) {
@@ -516,7 +544,7 @@ void rpmtsCleanProblems(rpmts ts)
     rpmtsi pi = rpmtsiInit(ts);
     while ((p = rpmtsiNext(pi, 0)) != NULL)
 	rpmteCleanProblems(p);
-    pi = rpmtsiFree(pi);
+    rpmtsiFree(pi);
 }
 
 void rpmtsClean(rpmts ts)
@@ -531,7 +559,7 @@ void rpmtsClean(rpmts ts)
     pi = rpmtsiInit(ts);
     while ((p = rpmtsiNext(pi, 0)) != NULL)
 	rpmteCleanDS(p);
-    pi = rpmtsiFree(pi);
+    rpmtsiFree(pi);
 
     tsmem->addedPackages = rpmalFree(tsmem->addedPackages);
 
@@ -711,19 +739,30 @@ struct selabel_handle * rpmtsSELabelHandle(rpmts ts)
     return NULL;
 }
 
-rpmRC rpmtsSELabelInit(rpmts ts, const char *path)
+rpmRC rpmtsSELabelInit(rpmts ts, int open_status)
 {
 #if WITH_SELINUX
+    const char * path = selinux_file_context_path();
+
     if (ts == NULL || path == NULL) {
 	return RPMRC_FAIL;
     }
 
+    if (open_status) {
+	selinux_status_close();
+	if (selinux_status_open(0) < 0) {
+	    return RPMRC_FAIL;
+	}
+    } else if (!selinux_status_updated() && ts->selabelHandle) {
+	return RPMRC_OK;
+    }
+
     struct selinux_opt opts[] = {
 	{SELABEL_OPT_PATH, path}
     };
 
     if (ts->selabelHandle) {
-	rpmtsSELabelFini(ts);
+	rpmtsSELabelFini(ts, 0);
     }
     ts->selabelHandle = selabel_open(SELABEL_CTX_FILE, opts, 1);
 
@@ -734,13 +773,16 @@ rpmRC rpmtsSELabelInit(rpmts ts, const char *path)
     return RPMRC_OK;
 }
 
-void rpmtsSELabelFini(rpmts ts)
+void rpmtsSELabelFini(rpmts ts, int close_status)
 {
 #if WITH_SELINUX
     if (ts && ts->selabelHandle) {
 	selabel_close(ts->selabelHandle);
 	ts->selabelHandle = NULL;
     }
+    if (close_status) {
+	selinux_status_close();
+    }
 #endif
 }
 
