@@ -70,12 +70,22 @@ int rpmVerifyFile(const rpmts ts, const rpmfi fi,
      */
     switch (rpmfiFState(fi)) {
     case RPMFILE_STATE_NETSHARED:
-    case RPMFILE_STATE_REPLACED:
     case RPMFILE_STATE_NOTINSTALLED:
-    case RPMFILE_STATE_WRONGCOLOR:
     case RPMFILE_STATE_MISSING:
 	return 0;
 	break;
+    case RPMFILE_STATE_REPLACED:
+	/* For replaced files we can only verify if it exists at all */
+	flags = RPMVERIFY_LSTATFAIL;
+	break;
+    case RPMFILE_STATE_WRONGCOLOR:
+	/*
+	 * Files with wrong color are supposed to share some attributes
+	 * with the actually installed file - verify what we can.
+	 */
+	flags &= ~(RPMVERIFY_FILEDIGEST | RPMVERIFY_FILESIZE |
+		   RPMVERIFY_MTIME | RPMVERIFY_RDEV);
+	break;
     case RPMFILE_STATE_NORMAL:
 	break;
     }
@@ -85,38 +95,23 @@ int rpmVerifyFile(const rpmts ts, const rpmfi fi,
 	return 1;
     }
 
-    /*
-     * Not all attributes of non-regular files can be verified.
-     */
-    if (S_ISDIR(sb.st_mode))
-	flags &= ~(RPMVERIFY_FILEDIGEST | RPMVERIFY_FILESIZE | RPMVERIFY_MTIME |
-			RPMVERIFY_LINKTO | RPMVERIFY_CAPS);
-    else if (S_ISLNK(sb.st_mode)) {
-	flags &= ~(RPMVERIFY_FILEDIGEST | RPMVERIFY_FILESIZE | RPMVERIFY_MTIME |
-			RPMVERIFY_MODE | RPMVERIFY_CAPS);
-    }
-    else if (S_ISFIFO(sb.st_mode))
-	flags &= ~(RPMVERIFY_FILEDIGEST | RPMVERIFY_FILESIZE | RPMVERIFY_MTIME |
-			RPMVERIFY_LINKTO | RPMVERIFY_CAPS);
-    else if (S_ISCHR(sb.st_mode))
-	flags &= ~(RPMVERIFY_FILEDIGEST | RPMVERIFY_FILESIZE | RPMVERIFY_MTIME |
-			RPMVERIFY_LINKTO | RPMVERIFY_CAPS);
-    else if (S_ISBLK(sb.st_mode))
-	flags &= ~(RPMVERIFY_FILEDIGEST | RPMVERIFY_FILESIZE | RPMVERIFY_MTIME |
-			RPMVERIFY_LINKTO | RPMVERIFY_CAPS);
-    else 
+    /* Links have no mode, other types have no linkto */
+    if (S_ISLNK(sb.st_mode))
+	flags &= ~(RPMVERIFY_MODE);
+    else
 	flags &= ~(RPMVERIFY_LINKTO);
 
-    /*
-     * Content checks of %ghost files are meaningless.
-     */
-    if (fileAttrs & RPMFILE_GHOST)
-	flags &= ~(RPMVERIFY_FILEDIGEST | RPMVERIFY_FILESIZE | RPMVERIFY_MTIME |
-			RPMVERIFY_LINKTO);
+    /* Not all attributes of non-regular files can be verified */
+    if (!S_ISREG(sb.st_mode))
+	flags &= ~(RPMVERIFY_FILEDIGEST | RPMVERIFY_FILESIZE |
+		   RPMVERIFY_MTIME | RPMVERIFY_CAPS);
 
-    /*
-     * Don't verify any features in omitMask.
-     */
+    /* Content checks of %ghost files are meaningless. */
+    if (fileAttrs & RPMFILE_GHOST)
+	flags &= ~(RPMVERIFY_FILEDIGEST | RPMVERIFY_FILESIZE |
+		   RPMVERIFY_MTIME | RPMVERIFY_LINKTO);
+
+    /* Don't verify any features in omitMask. */
     flags &= ~(omitMask | RPMVERIFY_FAILURES);
 
 
@@ -267,12 +262,18 @@ static int rpmVerifyScript(rpmts ts, Header h)
 
     if (headerIsEntry(h, RPMTAG_VERIFYSCRIPT)) {
 	/* fake up a erasure transaction element */
-	(void) rpmtsAddEraseElement(ts, h, -1);
+	rpmte p = rpmteNew(ts, h, TR_REMOVED, NULL, NULL);
 
-	rc = (rpmteProcess(rpmtsElement(ts, 0), PKG_VERIFY) != RPMRC_OK);
+	if (p != NULL) {
+	    rpmteSetHeader(p, h);
 
-	/* clean up our fake transaction bits */
-	rpmtsEmpty(ts);
+	    rc = (rpmpsmRun(ts, p, PKG_VERIFY) != RPMRC_OK);
+
+	    /* clean up our fake transaction bits */
+	    rpmteFree(p);
+	} else {
+	    rc = RPMRC_FAIL;
+	}
     }
 
     return rc;
@@ -291,14 +292,14 @@ char * rpmVerifyString(uint32_t verifyResult, const char *pad)
 {
     char *fmt = NULL;
     rasprintf(&fmt, "%s%s%s%s%s%s%s%s%s",
-		_verifyfile(RPMVERIFY_FILEDIGEST, "5", pad),
 		_verify(RPMVERIFY_FILESIZE, "S", pad),
-		_verifylink(RPMVERIFY_LINKTO, "L", pad),
-		_verify(RPMVERIFY_MTIME, "T", pad),
+		_verify(RPMVERIFY_MODE, "M", pad),
+		_verifyfile(RPMVERIFY_FILEDIGEST, "5", pad),
 		_verify(RPMVERIFY_RDEV, "D", pad),
+		_verifylink(RPMVERIFY_LINKTO, "L", pad),
 		_verify(RPMVERIFY_USER, "U", pad),
 		_verify(RPMVERIFY_GROUP, "G", pad),
-		_verify(RPMVERIFY_MODE, "M", pad),
+		_verify(RPMVERIFY_MTIME, "T", pad),
 		_verify(RPMVERIFY_CAPS, "P", pad));
 		
     return fmt;
@@ -338,6 +339,9 @@ static int verifyHeader(rpmts ts, Header h, rpmVerifyAttrs omitMask, int ghosts)
     int ec = 0;		/* assume no problems */
     rpmfi fi = rpmfiNew(ts, h, RPMTAG_BASENAMES, RPMFI_FLAGS_VERIFY);
 
+    if (fi == NULL)
+	return 1;
+
     rpmfiInit(fi, 0);
     while (rpmfiNext(fi) >= 0) {
 	rpmfileAttrs fileAttrs = rpmfiFFlags(fi);
@@ -424,9 +428,9 @@ static int verifyDependencies(rpmts ts, Header h)
 	    rpmlog(RPMLOG_NOTICE, "\t%s\n", ps);
 	    free(ps);
 	}
-	psi = rpmpsFreeIterator(psi);
+	rpmpsFreeIterator(psi);
     }
-    ps = rpmpsFree(ps);
+    rpmpsFree(ps);
     rpmtsEmpty(ts);
 
     return rc;
@@ -487,7 +491,7 @@ int rpmcliVerify(rpmts ts, QVA_t qva, char * const * argv)
     rpmtsSetScriptFd(ts, scriptFd);
     ovsflags = rpmtsSetVSFlags(ts, vsflags);
     ec = rpmcliArgIter(ts, qva, argv);
-    vsflags = rpmtsSetVSFlags(ts, ovsflags);
+    rpmtsSetVSFlags(ts, ovsflags);
     rpmtsSetScriptFd(ts, NULL);
 
     if (qva->qva_showPackage == showVerifyPackage)
