@@ -438,7 +438,7 @@ expandThis(MacroBuf mb, const char * src, size_t slen, char **target)
 static void mbAppend(MacroBuf mb, char c)
 {
     if (mb->nb < 1) {
-	mb->buf = xrealloc(mb->buf, mb->tpos + mb->nb + MACROBUFSIZ);
+	mb->buf = xrealloc(mb->buf, mb->tpos + MACROBUFSIZ + 1);
 	mb->nb += MACROBUFSIZ;
     }
     mb->buf[mb->tpos++] = c;
@@ -449,7 +449,7 @@ static void mbAppendStr(MacroBuf mb, const char *str)
 {
     size_t len = strlen(str);
     if (len > mb->nb) {
-	mb->buf = xrealloc(mb->buf, mb->tpos + mb->nb + MACROBUFSIZ + len);
+	mb->buf = xrealloc(mb->buf, mb->tpos + mb->nb + MACROBUFSIZ + len + 1);
 	mb->nb += MACROBUFSIZ + len;
     }
     memcpy(mb->buf+mb->tpos, str, len);
@@ -535,7 +535,7 @@ doDefine(MacroBuf mb, const char * se, int level, int expandbody)
 	    rpmlog(RPMLOG_ERR,
 		_("Macro %%%s has unterminated body\n"), n);
 	    se = s;	/* XXX W2DO? */
-	    return se;
+	    goto exit;
 	}
 	s++;	/* XXX skip { */
 	strncpy(b, s, (se - s));
@@ -573,7 +573,7 @@ doDefine(MacroBuf mb, const char * se, int level, int expandbody)
 	    rpmlog(RPMLOG_ERR,
 		_("Macro %%%s has unterminated body\n"), n);
 	    se = s;	/* XXX W2DO? */
-	    return se;
+	    goto exit;
 	}
 
 	/* Trim trailing blanks/newlines */
@@ -591,7 +591,7 @@ doDefine(MacroBuf mb, const char * se, int level, int expandbody)
     if (!((c = *n) && (risalpha(c) || c == '_') && (ne - n) > 2)) {
 	rpmlog(RPMLOG_ERR,
 		_("Macro %%%s has illegal name (%%define)\n"), n);
-	return se;
+	goto exit;
     }
 
     /* Options must be terminated with ')' */
@@ -691,16 +691,21 @@ pushMacro(rpmMacroEntry * mep,
 static void
 popMacro(rpmMacroEntry * mep)
 {
-	rpmMacroEntry me = (*mep ? *mep : NULL);
+    if (mep && *mep) {
+	rpmMacroEntry me = *mep;
 
-	if (me) {
-		/* XXX cast to workaround const */
-		if ((*mep = me->prev) == NULL)
-			me->name = _free(me->name);
-		me->opts = _free(me->opts);
-		me->body = _free(me->body);
-		me = _free(me);
-	}
+	/* restore previous definition of the macro */
+	*mep = me->prev;
+
+	/* name is shared between entries, only free if last of its kind */
+	if (me->prev == NULL)
+	    free(me->name);
+	free(me->opts);
+	free(me->body);
+
+	memset(me, 0, sizeof(*me)); /* trash and burn */
+	free(me);
+    }
 }
 
 /**
@@ -959,6 +964,15 @@ doFoo(MacroBuf mb, int negate, const char * f, size_t fn,
         case COMPRESSED_XZ:
             sprintf(be, "%%__xz -dc %s", b);
             break;
+	case COMPRESSED_LZIP:
+	    sprintf(be, "%%__lzip -dc %s", b);
+	    break;
+	case COMPRESSED_LRZIP:
+	    sprintf(be, "%%__lrzip -dqo- %s", b);
+	    break;
+	case COMPRESSED_7ZIP:
+	    sprintf(be, "%%__7zip x %s", b);
+	    break;
 	}
 	b = be;
     } else if (STREQ("getenv", f, fn)) {
@@ -1016,12 +1030,12 @@ expandMacro(MacroBuf mb, const char *src, size_t slen)
     char *source = NULL;
 
     /* Handle non-terminated substrings by creating a terminated copy */
-    if (slen > 0) {
-	source = xmalloc(slen + 1);
-	strncpy(source, src, slen);
-	source[slen] = '\0';
-	s = source;
-    }
+    if (!slen)
+	slen = strlen(src);
+    source = xmalloc(slen + 1);
+    strncpy(source, src, slen);
+    source[slen] = '\0';
+    s = source;
 
     if (mb->buf == NULL) {
 	size_t blen = MACROBUFSIZ + strlen(s);
@@ -1222,17 +1236,17 @@ expandMacro(MacroBuf mb, const char *src, size_t slen)
 		const char *ls = s+sizeof("{lua:")-1;
 		const char *lse = se-sizeof("}")+1;
 		char *scriptbuf = (char *)xmalloc((lse-ls)+1);
-		const char *printbuf;
+		char *printbuf;
 		memcpy(scriptbuf, ls, lse-ls);
 		scriptbuf[lse-ls] = '\0';
-		rpmluaSetPrintBuffer(lua, 1);
+		rpmluaPushPrintBuffer(lua);
 		if (rpmluaRunScript(lua, scriptbuf, NULL) == -1)
 		    rc = 1;
-		printbuf = rpmluaGetPrintBuffer(lua);
+		printbuf = rpmluaPopPrintBuffer(lua);
 		if (printbuf) {
 		    mbAppendStr(mb, printbuf);
+		    free(printbuf);
 		}
-		rpmluaSetPrintBuffer(lua, 0);
 		free(scriptbuf);
 		s = se;
 		continue;
@@ -1457,10 +1471,8 @@ rpmLoadMacroFile(rpmMacroContext mc, const char * fn)
     char *buf = xmalloc(blen);
     int rc = -1;
 
-    if (fd == NULL || ferror(fd)) {
-	if (fd) (void) fclose(fd);
+    if (fd == NULL)
 	goto exit;
-    }
 
     /* XXX Assume new fangled macro expansion */
     max_macro_depth = 16;
@@ -1525,19 +1537,12 @@ rpmFreeMacros(rpmMacroContext mc)
     if (mc == NULL) mc = rpmGlobalMacroContext;
 
     if (mc->macroTable != NULL) {
-	int i;
-	for (i = 0; i < mc->firstFree; i++) {
-	    rpmMacroEntry me;
-	    while ((me = mc->macroTable[i]) != NULL) {
-		/* XXX cast to workaround const */
-		if ((mc->macroTable[i] = me->prev) == NULL)
-		    me->name = _free(me->name);
-		me->opts = _free(me->opts);
-		me->body = _free(me->body);
-		me = _free(me);
+	for (int i = 0; i < mc->firstFree; i++) {
+	    while (mc->macroTable[i] != NULL) {
+		popMacro(&mc->macroTable[i]);
 	    }
 	}
-	mc->macroTable = _free(mc->macroTable);
+	free(mc->macroTable);
     }
     memset(mc, 0, sizeof(*mc));
 }
@@ -1599,7 +1604,7 @@ rpmExpandNumeric(const char *arg)
 	if (!(end && *end == '\0'))
 	    rc = 0;
     }
-    val = _free(val);
+    free(val);
 
     return rc;
 }
