@@ -960,27 +960,6 @@ unsigned int pgpGrab(const uint8_t *s, size_t nbytes)
     return i;
 }
 
-/** \ingroup rpmpgp
- * Return length of an OpenPGP packet.
- * @param s		pointer to packet
- * @retval *lenp	no. of bytes in packet
- * @return		no. of bytes in length prefix
- */
-static inline
-size_t pgpLen(const uint8_t *s, size_t * lenp)
-{
-    if (*s < 192) {
-	(*lenp) = *s++;
-	return 1;
-    } else if (*s < 255) {
-	(*lenp) = ((((unsigned)s[0]) - 192) << 8) + s[1] + 192;
-	return 2;
-    } else {
-	(*lenp) = pgpGrab(s+1, (size_t) 4);
-	return 5;
-    }
-}
-
 /** \ingroup rpmpgp
  * Return hex formatted representation of bytes.
  * @param p		bytes
@@ -1009,6 +988,17 @@ int pgpPubkeyFingerprint(const uint8_t * pkt, size_t pktlen,
 */
 int pgpExtractPubkeyFingerprint(const char * b64pkt, pgpKeyID_t keyid);
 
+/** \ingroup rpmpgp
+ * Parse a OpenPGP packet(s).
+ * @param pkts		OpenPGP packet(s)
+ * @param pktlen	OpenPGP packet(s) length (no. of bytes)
+ * @param pkttype	Expected packet type (signature/key) or 0 for any
+ * @retval ret		signature/pubkey packet parameters on success (alloced)
+ * @return		-1 on error, 0 on success
+ */
+int pgpPrtParams(const uint8_t *pkts, size_t pktlen, unsigned int pkttype,
+		 pgpDigParams * ret);
+
 /** \ingroup rpmpgp
  * Print/parse a OpenPGP packet(s).
  * @param pkts		OpenPGP packet(s)
@@ -1065,8 +1055,50 @@ void pgpCleanDig(pgpDig dig);
  */
 pgpDig pgpFreeDig(pgpDig dig);
 
+/** \ingroup rpmpgp
+ * Retrieve parameters for parsed OpenPGP packet(s).
+ * @param dig		container
+ * @param pkttype	type of params to retrieve (signature / pubkey)
+ * @return		pointer to OpenPGP parameters, NULL on error/not found
+ */
+pgpDigParams pgpDigGetParams(pgpDig dig, unsigned int pkttype);
+
+/** \ingroup rpmpgp
+ * Compare OpenPGP packet parameters
+ * param p1		1st parameter container
+ * param p2		2nd parameter container
+ * return		1 if the parameters differ, 0 otherwise
+ */
+int pgpDigParamsCmp(pgpDigParams p1, pgpDigParams p2);
+
+/** \ingroup rpmpgp
+ * Retrieve OpenPGP algorithm parameters
+ * param digp		parameter container
+ * param algotype	PGPVAL_HASHALGO / PGPVAL_PUBKEYALGO
+ * return		algorithm value, 0 on error
+ */
+unsigned int pgpDigParamsAlgo(pgpDigParams digp, unsigned int algotype);
+
+/** \ingroup rpmpgp
+ * Destroy parsed OpenPGP packet parameter(s).
+ * @param digp		parameter container
+ * @return		NULL always
+ */
+pgpDigParams pgpDigParamsFree(pgpDigParams digp);
+
 /** \ingroup rpmpgp
  * Verify a PGP signature.
+ * @param key		public key
+ * @param sig		signature
+ * @param hashctx	digest context
+ * @return 		RPMRC_OK on success 
+ */
+rpmRC pgpVerifySignature(pgpDigParams key, pgpDigParams sig, DIGEST_CTX hashctx);
+
+/** \ingroup rpmpgp
+ * Verify a PGP signature.
+ * @deprecated		use pgpVerifySignature() instead
+ *
  * @param dig		container
  * @param hashctx	digest context
  * @return 		RPMRC_OK on success 
